@@ -1,7 +1,7 @@
 from statsmodels.compat.numpy import lstsq
-from statsmodels.compat.pandas import assert_index_equal
+from statsmodels.compat.pandas import MONTH_END, YEAR_END, assert_index_equal
 from statsmodels.compat.platform import PLATFORM_WIN
-from statsmodels.compat.python import lrange
+from statsmodels.compat.python import PYTHON_IMPL_WASM, lrange
 
 import os
 import warnings
@@ -17,6 +17,7 @@ from numpy.testing import (
 import pandas as pd
 from pandas import DataFrame, Series, date_range
 import pytest
+from scipy import stats
 from scipy.interpolate import interp1d
 
 from statsmodels.datasets import macrodata, modechoice, nile, randhie, sunspots
@@ -25,6 +26,7 @@ from statsmodels.tools.sm_exceptions import (
     InfeasibleTestError,
     InterpolationWarning,
     MissingDataError,
+    ValueWarning,
 )
 # Remove imports when range unit root test gets an R implementation
 from statsmodels.tools.validation import array_like, bool_like
@@ -36,6 +38,7 @@ from statsmodels.tsa.stattools import (
     adfuller,
     arma_order_select_ic,
     breakvar_heteroskedasticity_test,
+    ccf,
     ccovf,
     coint,
     grangercausalitytests,
@@ -69,7 +72,15 @@ def acovf_data():
     return rnd.randn(250)
 
 
-class CheckADF(object):
+@pytest.fixture(scope="module")
+def gc_data():
+    mdata = macrodata.load_pandas().data
+    mdata = mdata[["realgdp", "realcons"]].values
+    data = mdata.astype(float)
+    return np.diff(np.log(data), axis=0)
+
+
+class CheckADF:
     """
     Test Augmented Dickey-Fuller
 
@@ -129,8 +140,6 @@ class TestADFNoConstant(CheckADF):
 
     @classmethod
     def setup_class(cls):
-        with pytest.warns(FutureWarning):
-            adfuller(cls.x, regression="nc", autolag=None, maxlag=4)
         cls.res1 = adfuller(cls.x, regression="n", autolag=None, maxlag=4)
         cls.teststat = 3.5227498
 
@@ -182,7 +191,13 @@ class TestADFNoConstant2(CheckADF):
         )
 
 
-class CheckCorrGram(object):
+@pytest.mark.parametrize("x", [np.full(8, 5.0)])
+def test_adfuller_resid_variance_zero(x):
+    with pytest.raises(ValueError):
+        adfuller(x)
+
+
+class CheckCorrGram:
     """
     Set up for ACF, PACF tests.
     """
@@ -334,6 +349,14 @@ class TestPACF(CheckCorrGram):
         pacfyw = pacf_yw(self.x, nlags=40, method="mle")
         assert_almost_equal(pacfyw[1:], self.pacfyw, DECIMAL_8)
 
+    @pytest.mark.skipif(
+        PYTHON_IMPL_WASM,
+        reason="No fp exception support in WASM"
+    )
+    def test_yw_singular(self):
+        with pytest.warns(ValueWarning):
+            pacf(np.ones(30), nlags=6)
+
     def test_ld(self):
         pacfyw = pacf_yw(self.x, nlags=40, method="mle")
         pacfld = pacf(self.x, nlags=40, method="ldb")
@@ -343,8 +366,42 @@ class TestPACF(CheckCorrGram):
         pacfld = pacf(self.x, nlags=40, method="lda")
         assert_almost_equal(pacfyw, pacfld, DECIMAL_8)
 
+    def test_burg(self):
+        pacfburg_, _ = pacf_burg(self.x, nlags=40)
+        pacfburg = pacf(self.x, nlags=40, method="burg")
+        assert_almost_equal(pacfburg_, pacfburg, DECIMAL_8)
 
-class TestBreakvarHeteroskedasticityTest(object):
+
+class TestCCF:
+    """
+    Test cross-correlation function
+    """
+
+    data = macrodata.load_pandas()
+    x = data.data["unemp"].diff().dropna()
+    y = data.data["infl"].diff().dropna()
+    filename = os.path.join(CURR_DIR, "results", "results_ccf.csv")
+    results = pd.read_csv(filename, delimiter=",")
+    nlags = 20
+
+    @classmethod
+    def setup_class(cls):
+        cls.ccf = cls.results['ccf']
+        cls.res1 = ccf(cls.x, cls.y, nlags=cls.nlags, adjusted=False, fft=False)
+
+    def test_ccf(self):
+        assert_almost_equal(self.res1, self.ccf, DECIMAL_8)
+
+    def test_confint(self):
+        alpha = 0.05
+        res2, confint = ccf(self.x, self.y, nlags=self.nlags, adjusted=False, fft=False, alpha=alpha)
+        assert_equal(res2, self.res1)
+        assert_almost_equal(res2 - confint[:, 0], confint[:, 1] - res2, DECIMAL_8)
+        alpha1 = stats.norm.cdf(confint[:, 1] - res2, scale=1.0 / np.sqrt(len(self.x)))
+        assert_almost_equal(alpha1, np.repeat(1 - alpha / 2.0, self.nlags), DECIMAL_8)
+
+
+class TestBreakvarHeteroskedasticityTest:
     from scipy.stats import chi2, f
 
     def test_1d_input(self):
@@ -464,7 +521,7 @@ class TestBreakvarHeteroskedasticityTest(object):
         assert actual_pvalue == expected_pvalue
 
 
-class CheckCoint(object):
+class CheckCoint:
     """
     Test Cointegration Test Results for 2-variable system
 
@@ -596,10 +653,6 @@ def test_coint():
     res[2] = [-1.6865000791270679, nan, nan, nan]
     res[3] = [-3.7991270451873675, nan, nan, nan]
 
-    with pytest.warns(FutureWarning):
-        # Ensure warning raised for nc rather than n
-        coint(y[:, 0], y[:, 1], trend="nc", maxlag=4, autolag=None)
-
     for trend in ["c", "ct", "ctt", "n"]:
         res1 = {}
         res1[0] = coint(y[:, 0], y[:, 1], trend=trend, maxlag=4, autolag=None)
@@ -655,7 +708,7 @@ def test_coint_perfect_collinearity():
     assert_(np.isneginf(c[0]))
 
 
-class TestGrangerCausality(object):
+class TestGrangerCausality:
     def test_grangercausality(self):
         # some example data
         mdata = macrodata.load_pandas().data
@@ -665,7 +718,8 @@ class TestGrangerCausality(object):
 
         # R: lmtest:grangertest
         r_result = [0.243097, 0.7844328, 195, 2]  # f_test
-        gr = grangercausalitytests(data[:, 1::-1], 2, verbose=False)
+        with pytest.warns(FutureWarning, match="verbose is"):
+            gr = grangercausalitytests(data[:, 1::-1], 2, verbose=False)
         assert_almost_equal(r_result, gr[2][0]["ssr_ftest"], decimal=7)
         assert_almost_equal(
             gr[2][0]["params_ftest"], gr[2][0]["ssr_ftest"], decimal=7
@@ -676,8 +730,10 @@ class TestGrangerCausality(object):
         mdata = mdata[["realgdp", "realcons"]].values
         data = mdata.astype(float)
         data = np.diff(np.log(data), axis=0)
-        gr = grangercausalitytests(data[:, 1::-1], 2, verbose=False)
-        gr2 = grangercausalitytests(data[:, 1::-1], [2], verbose=False)
+        with pytest.warns(FutureWarning, match="verbose is"):
+            gr = grangercausalitytests(data[:, 1::-1], 2, verbose=False)
+        with pytest.warns(FutureWarning, match="verbose is"):
+            gr2 = grangercausalitytests(data[:, 1::-1], [2], verbose=False)
         assert 1 in gr
         assert 1 not in gr2
         assert_almost_equal(
@@ -690,9 +746,11 @@ class TestGrangerCausality(object):
     def test_granger_fails_on_nobs_check(self, reset_randomstate):
         # Test that if maxlag is too large, Granger Test raises a clear error.
         x = np.random.rand(10, 2)
-        grangercausalitytests(x, 2, verbose=False)  # This should pass.
+        with pytest.warns(FutureWarning, match="verbose is"):
+            grangercausalitytests(x, 2, verbose=False)  # This should pass.
         with pytest.raises(ValueError):
-            grangercausalitytests(x, 3, verbose=False)
+            with pytest.warns(FutureWarning, match="verbose is"):
+                grangercausalitytests(x, 3, verbose=False)
 
     def test_granger_fails_on_finite_check(self, reset_randomstate):
         x = np.random.rand(1000, 2)
@@ -722,10 +780,9 @@ class TestKPSS:
     macrodata['realgdp'] series.
     """
 
-    @classmethod
-    def setup_class(cls):
-        cls.data = macrodata.load_pandas()
-        cls.x = cls.data.data["realgdp"].values
+    def setup_method(self):
+        self.data = macrodata.load_pandas()
+        self.x = self.data.data["realgdp"].values
 
     def test_fail_nonvector_input(self, reset_randomstate):
         # should be fine
@@ -843,10 +900,9 @@ class TestRUR:
     macrodata['realgdp'] series.
     """
 
-    @classmethod
-    def setup_class(cls):
-        cls.data = macrodata.load_pandas()
-        cls.x = cls.data.data["realgdp"].values
+    def setup_method(self):
+        self.data = macrodata.load_pandas()
+        self.x = self.data.data["realgdp"].values
 
     # To be removed when range unit test gets an R implementation
     def simple_rur(self, x, store=False):
@@ -857,7 +913,7 @@ class TestRUR:
 
         # if m is not one, n != m * n
         if nobs != x.size:
-            raise ValueError("x of shape {0} not understood".format(x.shape))
+            raise ValueError(f"x of shape {x.shape} not understood")
 
         # Table from [1] has been replicated using 200,000 samples
         # Critical values for new n_obs values have been identified
@@ -982,7 +1038,7 @@ def test_pandasacovf():
 
 def test_acovf2d(reset_randomstate):
     dta = sunspots.load_pandas().data
-    dta.index = date_range(start="1700", end="2009", freq="A")[:309]
+    dta.index = date_range(start="1700", end="2009", freq=YEAR_END)[:309]
     del dta["YEAR"]
     res = acovf(dta, fft=False)
     assert_equal(res, acovf(dta.values, fft=False))
@@ -1067,7 +1123,7 @@ def test_arma_order_select_ic():
     assert_(res.bic.index.equals(bic.index))
     assert_(res.bic.columns.equals(bic.columns))
 
-    index = pd.date_range("2000-1-1", freq="M", periods=len(y))
+    index = pd.date_range("2000-1-1", freq=MONTH_END, periods=len(y))
     y_series = pd.Series(y, index=index)
     res_pd = arma_order_select_ic(
         y_series, max_ar=2, max_ma=1, ic=["aic", "bic"], trend="n"
@@ -1381,7 +1437,7 @@ def test_adfuller_maxlag_too_large(reset_randomstate):
         adfuller(y, maxlag=51)
 
 
-class SetupZivotAndrews(object):
+class SetupZivotAndrews:
     # test directory
     cur_dir = CURR_DIR
     run_dir = os.path.join(cur_dir, "results")
@@ -1481,7 +1537,7 @@ def test_acf_conservate_nanops(reset_randomstate):
 
 
 def test_pacf_nlags_error(reset_randomstate):
-    e = np.random.standard_normal(100)
+    e = np.random.standard_normal(99)
     with pytest.raises(ValueError, match="Can only compute partial"):
         pacf(e, 50)
 
@@ -1518,4 +1574,41 @@ gc_data_sets = [df1, df2, df3, df4]
 @pytest.mark.parametrize("dataset", gc_data_sets)
 def test_granger_causality_exceptions(dataset):
     with pytest.raises(InfeasibleTestError):
-        grangercausalitytests(dataset, 4)
+        with pytest.warns(FutureWarning, match="verbose"):
+            grangercausalitytests(dataset, 4, verbose=False)
+
+
+def test_granger_causality_exception_maxlag(gc_data):
+    with pytest.raises(ValueError, match="maxlag must be"):
+        grangercausalitytests(gc_data, maxlag=-1)
+    with pytest.raises(NotImplementedError):
+        grangercausalitytests(gc_data, 3, addconst=False)
+
+
+def test_granger_causality_verbose(gc_data):
+    with pytest.warns(FutureWarning, match="verbose"):
+        grangercausalitytests(gc_data, 3, verbose=True)
+
+@pytest.mark.parametrize("size",[3,5,7,9])
+def test_pacf_small_sample(size,reset_randomstate):
+    y = np.random.standard_normal(size)
+    a = pacf(y)
+    assert isinstance(a, np.ndarray)
+    a, b = pacf_burg(y)
+    assert isinstance(a, np.ndarray)
+    assert isinstance(b, np.ndarray)
+    a = pacf_ols(y)
+    assert isinstance(a, np.ndarray)
+    a = pacf_yw(y)
+    assert isinstance(a, np.ndarray)
+
+
+def test_pacf_1_obs(reset_randomstate):
+    y = np.random.standard_normal(1)
+    with pytest.raises(ValueError):
+        pacf(y)
+    with pytest.raises(ValueError):
+        pacf_burg(y)
+    with pytest.raises(ValueError):
+        pacf_ols(y)
+    pacf_yw(y)
